

let ANIM_IDLE = "Idle";
let ANIM_MOVE = "Move";
let ANIM_MOVE_RIGHT = "MoveRight";
let ANIM_MOVE_LEFT = "MoveLeft";
let ANIM_MOVEEND = "MoveToIdle";
let ANIM_MOVEEND_RIGHT = "MoveRightToIdle";
let ANIM_MOVEEND_LEFT = "MoveLeftToIdle";
let ANIM_ATTACK = "Attack";
let ANIM_ATTACKEND = "AttackToIdle";

let AnimSrc;
let quality_a = GetCommonData("quality", 2);
local{AnimSrc = GetCurrentScriptDirectory;}
let img_effect_a = AnimSrc~"img/effect.png"; // defunct
let img_effect_anim = AnimSrc~"img/eff_aura.png";

let fl = false;

/*
Update:
 Change system to state based, so the user can force animations as well
 
  - Allow unlimited attack animations
  - Allow unlimited arbitrary animations
  - Allow the user to flip the image in an easier way
    - Change it to work only one frame instead of continuously
	- Allow the user to retreive the current (animation) scale
	- Allow the user to change the scale with a custom function
  - Allow the user to change the animation to any state
  - Allow the user to change the set of animations for the auto animations
  - Custom defined states will be "Attack"~AtkNum, and "Animation"~AnimNum
    - Predefined will remain the same
  - Change the animation to always display 1 array and take its properties that way
    - so instead of using if statements to determine which array to use,
	- there will always be a setrect every frame using the array
  - Ensure all functions check if the object exists

Fix:
  - Error when the boss does not exist and the animation is started
  - Ensure all functions check if the object exists

Perform error check:
  - When user is using the A2 variant, but uses ANIM_MOVE when setting loop point,
	 there is no proper error


*/




//normal release functions

function ObjAnim_AddFrameA1(objID, type, left, top, right, bot){
	let aarr = Obj_GetValueD(objID, "anim_"~type, []);
	Obj_SetValue(objID, "anim_"~type, aarr~[[left, top, right, bot, 0, 0]]);
}

function ObjAnim_AddFrameA2(objID, type, left, top, right, bot, xoff, yoff){
	let aarr = Obj_GetValueD(objID, "anim_"~type, []);
	Obj_SetValue(objID, "anim_"~type, aarr~[[left, top, right, bot, xoff, yoff]]);
}

function ObjAnim_ReplaceFrameA2(objID, type, animindex, left, top, right, bot, xoff, yoff){
	let aarr = Obj_GetValueD(objID, "anim_"~type, []);
	aarr[animindex] = [left, top, right, bot, xoff, yoff];
	Obj_SetValue(objID, "anim_"~type, aarr);
}


function ObjAnim_SetLoopPointA1(objID, type, frame){
	Obj_SetValue(objID, "anim_"~type~"_loop", frame);
}

function ObjAnim_SetSpeedA1(objID, type, speed){
	Obj_SetValue(objID, "anim_"~type~"_speed", speed);
}

function ObjAnim_FlipMoveImage(objID, cond){
	fl = cond;
}

task ObjAnim_ForceStop(objID){
	Obj_SetValue(objID, "anim_forcestop", true);
	yield;
	Obj_SetValue(objID, "anim_forcestop", false);
}

function ObjAnim_SetAttackA1(objID, status){
	if(Obj_IsDeleted(objID)){return;}
	DidUserSetFrames(objID, ANIM_ATTACK, "Attack");
	DidUserSetFrames(objID, ANIM_ATTACKEND, "Attack");
	if(!Obj_IsValueExists(objID, "anim_Attack_loop")){
		RaiseError("Did not specify loop value for attack animation");
	}
	Obj_SetValue(objID, "anim_playattack", status);
}

task ObjAnim_SetAttackA2(objID, length){
	if(Obj_IsDeleted(objID)){return;}
	DidUserSetFrames(objID, ANIM_ATTACK, "Attack");
	DidUserSetFrames(objID, ANIM_ATTACKEND, "Attack To Idle (ATTACKEND)");
	if(!Obj_IsValueExists(objID, "anim_Attack_loop")){
		RaiseError("Did not specify loop value for attack animation");
	}
	Obj_SetValue(objID, "anim_playattack", true);
	loop(length){yield;}
	Obj_SetValue(objID, "anim_playattack", false);
}

function ObjAnim_IsAttackPlaying(objID){
	return Obj_GetValueD(objID, "anim_isattackplaying", false);
}

task ObjAnim_StartAnimationA1(objID, bob, bobspeed){
	DidUserSetFrames(objID, ANIM_IDLE, "Idle");
	DidUserSetFrames(objID, ANIM_MOVE, "Move");
	DidUserSetFrames(objID, ANIM_MOVEEND, "Move To Idle (MOVEEND)");
	DidUserSetValidLoop(objID, ANIM_IDLE, "");
	DidUserSetValidLoop(objID, ANIM_MOVE, "");
	if(Obj_IsValueExists(objID, "anim_Attack_loop")){DidUserSetValidLoop(objID, ANIM_ATTACK, "");}
	
	let anim_count = 0;
	let anim_type = 0;
	let anim_rend = 0;
	
	let idle = Obj_GetValueD(objID, "anim_"~ANIM_IDLE~"_speed", 3);
	let move = Obj_GetValueD(objID, "anim_"~ANIM_MOVE~"_speed", 4);
	let attack = Obj_GetValueD(objID, "anim_"~ANIM_ATTACK~"_speed", 6);
	
	let AnimAttack = false;
	
	let x = 0;
	let y = 0;
	let fr = [];
	
	let m = 0;
	
	Obj_SetValue(objID, "anim_isattackplaying", false);
	
	while(!Obj_IsDeleted(objID)&&!Obj_GetValueD(objID, "anim_forcestop", false)){
		AnimAttack = Obj_GetValueD(objID, "anim_playattack", false);
		if(!AnimAttack){
			if(ObjMove_GetSpeed(objID) == 0){
				if(anim_type == 1){
					anim_count = 0;
					anim_type = 2;
				}
			}
			if(ObjMove_GetSpeed(objID) > 0){
				if(anim_type == 0){
					anim_count = 0;
					anim_type = 1;
				}
			}
			if(anim_type == 3){
				anim_type=4;
				anim_count = 0;
			}
		}else{
			Obj_SetValue(objID, "anim_isattackplaying", true);
			if(anim_type != 3){
				anim_type=3;
				anim_count = 0;
			}
		}
		
		if(CheckDirectionAL1(objID, "right")){ObjRender_SetScaleXYZ(objID, 1-2*fl, 1, 1)}
		if(CheckDirectionAL1(objID, "left")){ObjRender_SetScaleXYZ(objID, -1+2*fl, 1, 1)}
		
			
		if(anim_type==1){ //Moving
			anim_count+=1/move;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_MOVE))-1){anim_count = Obj_GetValue(objID, "anim_"~ANIM_MOVE~"_loop");}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_MOVE);
		}
		if(anim_type==2){ //Moving to idle
			anim_count+=1/move;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_MOVEEND))-1){anim_count = 0;anim_type=0;}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_MOVEEND);
		}
		if(anim_type==3){ //Attack
			anim_count+=1/attack;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_ATTACK))-1){anim_count = Obj_GetValue(objID, "anim_"~ANIM_ATTACK~"_loop");}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_ATTACK);
		}
		if(anim_type==4){ //Attack to idle
			anim_count+=1/attack;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_ATTACKEND))-1){anim_count = 0;anim_type=0;Obj_SetValue(objID, "anim_isattackplaying", false);}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_ATTACKEND);
		}
		if(anim_type==0){ //Idle
			anim_count+=1/idle;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_IDLE))-1){anim_count = Obj_GetValueD(objID, "anim_"~ANIM_IDLE~"_loop", 0);}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_IDLE);
		}
		x = fr[2]-fr[0];
		y = fr[3]-fr[1];
		
		m+=bobspeed;
		ObjSprite2D_SetSourceRect(objID, fr[0], fr[1], fr[2], fr[3]);
		ObjSprite2D_SetDestRect(objID, -x/2+fr[4]/2, -y/2+(bob*sin(m))+fr[5]/2, x/2+fr[4]/2, y/2+(bob*sin(m))+fr[5]/2);
		yield;
	}
	function EmergencyCheck(container){
		if(Obj_IsDeleted(objID)){break;}
		return Obj_GetValue(objID, "anim_"~container)[anim_rend];
	}	
}

task ObjAnim_StartAnimationA2(objID, bob, bobspeed){
	DidUserSetFrames(objID, ANIM_IDLE, "Idle");
	DidUserSetFrames(objID, ANIM_MOVE_RIGHT, "Move (right)");
	DidUserSetFrames(objID, ANIM_MOVE_LEFT, "Move (left)");
	DidUserSetFrames(objID, ANIM_MOVEEND_RIGHT, "Move To Idle (MOVEEND_RIGHT)");
	DidUserSetFrames(objID, ANIM_MOVEEND_LEFT, "Move To Idle (MOVEEND_LEFT)");
	DidUserSetValidLoop(objID, ANIM_IDLE, "");
	DidUserSetValidLoop(objID, ANIM_MOVE_RIGHT, " (right)");
	DidUserSetValidLoop(objID, ANIM_MOVE_LEFT, " (left)");
	if(Obj_IsValueExists(objID, "anim_Attack_loop")){DidUserSetValidLoop(objID, ANIM_ATTACK, "");}
	
	let anim_count = 0;
	let anim_type = 0;
	let anim_rend = 0;
	
	let idle = Obj_GetValueD(objID, "anim_"~ANIM_IDLE~"_speed", 3);
	let mover = Obj_GetValueD(objID, "anim_"~ANIM_MOVE_RIGHT~"_speed", 4);
	let movel = Obj_GetValueD(objID, "anim_"~ANIM_MOVE_LEFT~"_speed", 4);
	let attack = Obj_GetValueD(objID, "anim_"~ANIM_ATTACK~"_speed", 6);
	
	let AnimAttack = false;
	
	let x = 0;
	let y = 0;
	let fr = [];
	
	let m = 0;
	let LastDirection;
			
	Obj_SetValue(objID, "anim_isattackplaying", false);
	
	while(!Obj_IsDeleted(objID)&&!Obj_GetValueD(objID, "anim_forcestop", false)){
		if(Obj_IsDeleted(objID)){return;}
		AnimAttack = Obj_GetValueD(objID, "anim_playattack", false);
		if(!AnimAttack){
			if(ObjMove_GetSpeed(objID) == 0){
				if(anim_type == 1){
					anim_count = 0;
					anim_type = 2;
				}
			}
			if(ObjMove_GetSpeed(objID) > 0){
				if(anim_type == 0){
					anim_count = 0;
					anim_type = 1;
				}
			}
			if(anim_type == 3){
				anim_type=4;
				anim_count = 0;
			}
		}else{
			Obj_SetValue(objID, "anim_isattackplaying", true);
			if(anim_type != 3){
				anim_type=3;
				anim_count = 0;
			}
		}
		
		if(CheckDirectionAL1(objID, "right")){LastDirection = "Right";}
		if(CheckDirectionAL1(objID, "left")){LastDirection = "Left";}
		
			
		if(anim_type==1){ //Moving
			if(LastDirection == "Right"){
				if(Obj_IsDeleted(objID)){return;}
				anim_count+=1/mover;
				if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_MOVE_RIGHT))-1){anim_count = Obj_GetValue(objID, "anim_"~ANIM_MOVE_RIGHT~"_loop");}
				anim_rend = floor(anim_count);
				fr = EmergencyCheck(ANIM_MOVE_RIGHT);
			}
			else if(LastDirection == "Left"){
				if(Obj_IsDeleted(objID)){return;}
				anim_count+=1/movel;
				if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_MOVE_LEFT))-1){anim_count = Obj_GetValue(objID, "anim_"~ANIM_MOVE_LEFT~"_loop");}
				anim_rend = floor(anim_count);
				fr = EmergencyCheck(ANIM_MOVE_LEFT);
			}
			else{RaiseError("An error occurred when animating. If you cannot resolve the issue, please send the affected script to gtbot so he may fix it.");}
		}
		if(anim_type==2){ //Moving to idle
			if(LastDirection == "Right"){
				if(Obj_IsDeleted(objID)){return;}
				anim_count+=1/mover;
				if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_MOVEEND_RIGHT))-1){anim_count = 0;anim_type=0;}
				anim_rend = floor(anim_count);
				fr = EmergencyCheck(ANIM_MOVEEND_RIGHT);
			}
			else if(LastDirection == "Left"){
				if(Obj_IsDeleted(objID)){return;}
				anim_count+=1/movel;
				if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_MOVEEND_LEFT))-1){anim_count = 0;anim_type=0;}
				anim_rend = floor(anim_count);
				fr = EmergencyCheck(ANIM_MOVEEND_LEFT);
			}
			else{RaiseError("An error occurred when animating. If you cannot resolve the issue, please send the affected script to gtbot so he may fix it.");}
		}
		if(anim_type==3){ //Attack
			if(Obj_IsDeleted(objID)){return;}
			anim_count+=1/attack;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_ATTACK))-1){anim_count = Obj_GetValue(objID, "anim_"~ANIM_ATTACK~"_loop");}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_ATTACK);
		}
		if(anim_type==4){ //Attack to idle
			if(Obj_IsDeleted(objID)){return;}
			anim_count+=1/attack;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_ATTACKEND))-1){anim_count = 0;anim_type=0;Obj_SetValue(objID, "anim_isattackplaying", false);}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_ATTACKEND);
		}
		if(anim_type==0){ //Idle
			if(Obj_IsDeleted(objID)){return;}
			anim_count+=1/idle;
			if(floor(anim_count)>length(Obj_GetValue(objID, "anim_"~ANIM_IDLE))-1){anim_count = Obj_GetValueD(objID, "anim_"~ANIM_IDLE~"_loop", 0);}
			anim_rend = floor(anim_count);
			fr = EmergencyCheck(ANIM_IDLE);
		}
		x = fr[2]-fr[0];
		y = fr[3]-fr[1];
		
		m+=bobspeed;
		ObjSprite2D_SetSourceRect(objID, fr[0], fr[1], fr[2], fr[3]);
		ObjSprite2D_SetDestRect(objID, -x/2+fr[4]/2, -y/2+(bob*sin(m))+fr[5]/2, x/2+fr[4]/2, y/2+(bob*sin(m))+fr[5]/2);
		yield;
	}
	function EmergencyCheck(container){
		if(Obj_IsDeleted(objID)){break;}
		return Obj_GetValue(objID, "anim_"~container)[anim_rend];
	}	
}



function GetAnimationFrames(objID, type){
	return Obj_GetValue(objID, "anim_"~type);
}

function DidUserSetFrames(objID, type, str){
	if(!(length(Obj_GetValueD(objID, "anim_"~type, []))>0)){
		RaiseError("Did not specify frames for "~str~" Animation");
	}
}

function DidUserSetValidLoop(objID, type, str){
	if(!Obj_IsValueExists(objID, "anim_"~type)){
		RaiseError("Did not specify loop value for movement ("~str~") animation");
	}
	let loopvar = Obj_GetValueD(objID, "anim_"~type~"_loop", 0);
	let loopnum = length(Obj_GetValue(objID, "anim_"~type))-1;
	if(loopvar>loopnum){
		RaiseError("Specified loop value of "~IntToString(loopvar)~" for anim_"~type~"_loop "~str~" exceeds total number of frame indices: "~IntToString(loopnum));
	}
	else if(loopvar<0){
		RaiseError("Specified loop value of "~IntToString(loopvar)~" for anim_"~type~"_loop "~str~" is invalid: cannot be less than 0");
	}
}

function CheckDirectionAL1(checkid, direc){
	let direction = false;
	let enemyang = absolute(ObjMove_GetAngle(checkid));
	let enemyspd = absolute(ObjMove_GetSpeed(checkid));
	
	
	while(enemyang>360){
		enemyang = enemyang-360;
	}

	if(enemyspd > 0){
		if(direc=="right"){
			if(enemyang > 270 || enemyang-270 < 90){
				direction=true;
			}
		}
		if(direc=="left"){
			if(enemyang < 270 && enemyang > 90){
				direction=true;
			}
		}
	}
	if(direc=="none"){
		if(enemyspd == 0){
			direction=true;
		}
	}
	
	return direction;
}

function Animate(anim_count){
	return max(0, floor(anim_count));
}



task AuraOnBossSingleAnim(objid, r, g, b){	
	loop(4){yield;}
	if(quality_a==0){return;}
	let shift = [0, 120, 240];
	let color = [0, 0, 0];
	if(b != 0){color = [r,g,b];}
	let fe = 0;
	while(!Obj_IsDeleted(objid)){
		if(b==0){
			ascent(i in 0..3){
				shift[i] = shift[i]+2;
				color[i] = sin(shift[i])*128+127;
			}
		}
		if(fe>=9-quality_a){
			EffectAu1(ObjRender_GetX(objid), ObjRender_GetY(objid), 0);
			EffectAu1(ObjRender_GetX(objid), ObjRender_GetY(objid), 1);
			EffectAu1(ObjRender_GetX(objid), ObjRender_GetY(objid), 1);
			fe = 0;
		}
		fe+=1;
		yield;
	}	
	task EffectAu1(x, y, type){
		let oy = y;
		let dalpha = rand(155, 205);
		let obj = ObjPrim_Create(OBJ_SPRITE_2D);
		ObjRender_SetBlendType(obj, BLEND_ADD_ARGB);
		Obj_SetRenderPriority(obj, 0.31);
		ObjPrim_SetTexture(obj, img_effect_anim);
		ObjSprite2D_SetSourceRect(obj, (type)*48, 0, (type+1)*48, 64);
		ObjSprite2D_SetDestCenter(obj);
		ObjRender_SetScaleXYZ(obj, rand(1, 1.65), rand(1, 1.95), 1);
		ObjRender_SetAngleZ(obj, rand(0, 360));
		let rang = 90;
		if(type==0){
			ObjRender_SetAngleZ(obj, rand(-20, 20));
		}else{rang = rand(0,360);}
		ObjRender_SetColor(obj, color[0], color[1], color[2]);
		let alpha = 0;
		while(alpha < dalpha){
			alpha = max(alpha+16, dalpha);
			RenderMoving;
			yield;
		}
		while(alpha>0){
			alpha = max(alpha-9, 0);
			RenderMoving;
			yield;
		}
		Obj_Delete(obj);
		
		sub RenderMoving{
			x = x-((1-type)*2.3+0.85)*cos(rang);
			y = y-((1-type)*2.3+0.85)*sin(rang);
			ObjRender_SetPosition(obj, x, y, 0);
			ObjRender_SetAlpha(obj, alpha);
		}
	}
}


task NameOnBossSingleAnim(objid, text){
	let quality = GetCommonData("quality", 2);
	if(quality==0){return}
	let objText = ObjText_Create;
	ObjText_SetFontSize(objText, 11);
	ObjText_SetFontColorTop(objText, 120, 220, 154);
	ObjText_SetFontColorBottom(objText, 120, 220, 154);
	
	ObjText_SetHorizontalAlignment(objText, ALIGNMENT_LEFT);
	ObjText_SetMaxWidth(objText, 999);
	Obj_SetRenderPriority(objText, 0.79);
	ObjRender_SetX(objText, 1);
	ObjRender_SetY(objText, 1);
	ObjText_SetText(objText, text);
	if(GetCommonData("lifebar", "DDC") != "DDC"){
		ObjRender_SetX(objText, 8);
		ObjRender_SetY(objText, 11);
	}
	
	
	while(!Obj_IsDeleted(objid)){
		yield;
	}
	Obj_Delete(objText);
}






